'use strict';

var web3_js = require('@solana/web3.js');

class Sequential {
    connections;
    next = 0;
    constructor(connections) {
        this.connections = connections;
    }
    start() {
        this.next = 0;
    }
    *getConnection() {
        while (true) {
            if (this.next > this.connections.length - 1) {
                return null;
            }
            const con = this.connections[this.next];
            this.next++;
            yield con;
        }
    }
}

class RoundRobin {
    connections;
    next = 0;
    triedConnections = 0;
    constructor(connections) {
        this.connections = connections;
    }
    start() {
        this.triedConnections = 0;
    }
    *getConnection() {
        while (true) {
            // just cycles back to start when all connections are tried
            if (this.next > this.connections.length - 1) {
                this.next = 0;
            }
            if (this.triedConnections > this.connections.length - 1) {
                return null;
            }
            const con = this.connections[this.next];
            this.next++;
            this.triedConnections++;
            yield con;
        }
    }
}

class Random {
    connections;
    available = [];
    constructor(connections) {
        this.connections = connections;
    }
    start() {
        this.available = Array.from(Array(this.connections.length).keys());
    }
    *getConnection() {
        while (true) {
            if (this.available.length === 0) {
                return null;
            }
            const rand = Math.floor(Math.random() * this.available.length);
            const randomKey = this.available[rand];
            const con = this.connections[randomKey];
            this.available.splice(rand, 1);
            yield con;
        }
    }
}

class AdvancedConnection extends web3_js.Connection {
    connections;
    strategy;
    overrides;
    constructor(endpoints, commitmentOrConfig, advancedConfig) {
        // basically don't care about super
        super(endpoints[0] || "", commitmentOrConfig);
        // store connections
        this.connections = endpoints.map((url) => new web3_js.Connection(url, commitmentOrConfig));
        switch (advancedConfig?.strategy ?? 'sequential') {
            case "round-robin":
                this.strategy = new RoundRobin(this.connections);
                break;
            case "random":
                this.strategy = new Random(this.connections);
                break;
            default:
                this.strategy = new Sequential(this.connections);
                break;
        }
        this.overrides = new Map();
        advancedConfig?.routes?.forEach((route) => {
            let foundConnection = this.connections.find((con) => con.rpcEndpoint === route.endpoint);
            if (!foundConnection) {
                foundConnection = new web3_js.Connection(route.endpoint, commitmentOrConfig);
            }
            this.overrides.set(route.method, { allowFallback: route.allowFallback, connection: foundConnection });
        });
        // keep reference to this
        const self = this;
        for (const property of Object.getOwnPropertyNames(web3_js.Connection.prototype)) {
            // @ts-ignore
            if (typeof web3_js.Connection.prototype[property] !== 'function') {
                continue;
            }
            // Remap all functions with a proxy function that does the exact same thing,
            // except it adds a fallback for when something goes wrong
            // @ts-ignore
            if (this[property].constructor.name === 'AsyncFunction') {
                // @ts-ignore
                this[property] = async function (...args) {
                    return await self.executeWithCallback((con) => {
                        // @ts-ignore
                        return con[property].apply(con, args);
                    }, property);
                };
                continue;
            }
            // Do the same for non async functions
            // @ts-ignore
            this[property] = function (...args) {
                let lastError;
                // overrides come first, if set
                if (self.overrides.has(property)) {
                    const override = self.overrides.get(property);
                    if (override) {
                        try {
                            // @ts-ignore
                            return override.connection[property].apply(override.connection, args);
                        }
                        catch (e) {
                            lastError = e;
                        }
                        if (!override.allowFallback) {
                            if (lastError) {
                                throw lastError;
                            }
                        }
                    }
                }
                self.strategy.start();
                for (const conn of self.strategy.getConnection()) {
                    try {
                        // @ts-ignore
                        return conn[property].apply(conn, args);
                    }
                    catch (e) {
                        lastError = e;
                    }
                }
                // re-throw last error
                if (lastError) {
                    throw lastError;
                }
            };
        }
    }
    executeWithCallback = async (callback, property) => {
        // start with main connection, then iterate through all backups
        let lastError;
        // overrides come first, if set
        if (this.overrides.has(property)) {
            const override = this.overrides.get(property);
            if (override) {
                try {
                    return await callback(override.connection);
                }
                catch (e) {
                    lastError = e;
                }
                if (!override.allowFallback) {
                    if (lastError) {
                        throw lastError;
                    }
                }
            }
        }
        this.strategy.start();
        for (const conn of this.strategy.getConnection()) {
            try {
                return await callback(conn);
            }
            catch (e) {
                lastError = e;
            }
        }
        // if we went through all connections and it's still failing, throw the last error
        throw lastError;
    };
}

module.exports = AdvancedConnection;
//# sourceMappingURL=main.cjs.js.map
